import logging
import ldap
import psycopg2
from psycopg2 import sql
from oscars_etl.users import UserAttribute

LOGGER = logging.getLogger(__name__)


def username2email(
        ldap_username=None,
        ldap_password=None,
        ldap_url='ipa01.vgt.vito.be',
        db_host='postgresqldmz2.services.rscloud.vito.be',
        db_name='cvbdmz2',
        db_username=None,
        db_password=None
):
    try:
        db_conn = psycopg2.connect(f"host='{db_host}' dbname='{db_name}' user='{db_username}' password='{db_password}'")
    except Exception as e:
        LOGGER.error(f"Could not connect to database: {e}")
        raise e

    ldap_conn = ldap.initialize(ldap_url)
    ldap_conn.simple_bind_s(ldap_username, ldap_password)

    with db_conn.cursor() as cursor, db_conn.cursor() as write_cursor:
        cursor.execute(sql.SQL("SELECT DISTINCT username FROM oscars_stats WHERE email IS NULL"))
        for (username,) in cursor:
            email = get_user_info(ldap_conn, username)
            LOGGER.info(f"{username}: {email}")
            if email is not None:
                write_cursor.execute(sql.SQL(f"""
                    UPDATE oscars_stats 
                    SET email = %(email)s
                    WHERE username = %(username)s AND email IS NULL
                    """), {"email": email, "username": username})
            else:
                LOGGER.warning(f"Could not find email for user {username}")

    db_conn.commit()
    ldap_conn.unbind()
    db_conn.close()


def get_user_info(ldap_conn, username):
    r = ldap_conn.search(
        base=f'cn=users,cn=accounts,dc=vgt,dc=vito,dc=be',
        scope=ldap.SCOPE_ONELEVEL,
        filterstr=f"(&(uid={username})(objectClass=MEPUser))",
        attrlist=[UserAttribute.UID.ldap_attribute, UserAttribute.EMAIL.ldap_attribute]
    )
    _, result = ldap_conn.result(r)
    if len(result) == 1:
        user = result[0][1]
        return user[UserAttribute.EMAIL.ldap_attribute][0].decode('utf8') \
            if UserAttribute.EMAIL.ldap_attribute in user \
            else None
    else:
        return None


if __name__ == "__main__":
    import os
    import logging.config

    _basedir = os.path.dirname(os.path.realpath(__file__))
    logging.config.fileConfig(os.path.join(_basedir, 'logging.conf'),
                              disable_existing_loggers=False)

    import fire

    fire.Fire(username2email)
